#ifndef __cplusplus
#error Must compile using C++ for using Picture class.
#endif

#ifndef __PICTURE_HPP
#define __PICTURE_HPP

#include <alloc.h>
#include <fstream.h>
#include <string.h>
#include <mem.h>
#include "include\vmode.h"
#include "include\palette.h"

// REM out this line if your compiler supports bool or _bool.
#include "include\boolean.h"

// 128 byte header info for PCX files...
// According to documentation by ZSoft...
struct PCX_File_Header {
    char manufacturer;      // Constant Flag, 10 = ZSoft .pcx
    char version;           // Version information
                            //    0 = Version 2.5 of PC Paintbrush
                            //    2 = Version 2.8 w/palette information
                            //    3 = Version 2.8 w/o palette information
                            //    4 = PC Paintbrush for Windows(Plus for
                            //        Windows uses Ver 5)
                            //    5 = Version 3.0 and > of PC Paintbrush
                            //        and PC Paintbrush +, includes
                            //        Publisher's Paintbrush . Includes
                            //        24-bit .PCX files
    char encoding;          // 1 = .PCX run length encoding
    char bitsPerPixel;      // Number of bits to represent a pixel
                            //        (per Plane) - 1, 2, 4, or 8
    int xMin, yMin;         // Window: Xmin,Ymin
    int xMax, yMax;         // Window: Xmax,Ymax
    int horizRes;           // Horizontal Resolution of image in DPI*
    int vertRes;            // Vertical Resolution of image in DPI*
    char egaPalette[16][3]; // Color palette setting, see text
    char reserved;          // Should be set to 0.
    char numOfPlanes;       // Number of color planes
    int bytesPerLine;       // Number of bytes to allocate for a scanline
                            //        plane.  MUST be an EVEN number.  Do NOT
                            //        calculate from Xmax-Xmin.
    int paletteType;        // How to interpret palette- 1 = Color/BW,
                            //        2 = Grayscale (ignored in PB IV/ IV +)
    int hScreenSize;        // Horizontal screen size in pixels.
                            //       New field found only in PB IV/IV Plus
    int vScreenSize;        // Vertical screen size in pixels. New field
                            //       found only in PB IV/IV Plus
    char filler[54];        // Blank to fill out 128 byte header.  Set all
                            //       bytes to 0
};

// Bitmap file header according to BMP documentation...
struct BMP_File_Header {
    struct {
        char fileType[2];
        long fileSize;
        int reserved1;
        int reserved2;
        long picOffset;
    } File;
    union {
        struct {
            long size;
            long picWidth;
            long picHeight;
            int numOfPlanes;
            int bitsPerPixel;
            long compression;
            long imageSize;
            long xPelsPerMeter;
            long yPelsPerMeter;
            long colorsUsed;
            long importantColors;
        } Windows;
        struct {
            long size;
            int picWidth;
            int picHeight;
            int numOfPlanes;
            int bitsPerPixel;
        } OS2;
    } Info;
};

class Picture {
    union {
        struct PCX_File_Header PCX;
        struct BMP_File_Header BMP;
    } header;
    union ColorAttribute palette[ScreenColors];
    char fileFormat[4];
    char subFormat;

    unsigned int width, height;
    unsigned char far *image;

    unsigned char far *displayMem;

    bool initOk;

public:
    Picture(int w=0, int h=0, bool useVideoMem=true);
    ~Picture(void);

    bool InitOk(void) {return initOk;}

    bool LoadImageFromFile(const char *fileName);
    bool SaveImageToFile(const char *fileName);

    void DisplayImage(int xOffset=0, int yOffset=0);
    void DisplayImagePalette(void)
    {
        SetPaletteUsing(palette);
    }

private:
    void InitPCXHeader(void);
    bool WritePCXFile(ofstream &fout);
    bool ReadPCXFile(ifstream &fin);
};

#endif  /* __PICTURE_HPP */
