/******************************************************************************
 *                             BASIC SPRITE CLASS                             *
 *                              by Gary Neal, Jr.                             *
 *               garyneal_71@yahoo.com       garyneal@geocities.com           *
 ******************************************************************************/

#ifndef _SPRITE_HPP
#define _SPRITE_HPP

#ifndef __cplusplus
#error The Basic Sprite Class defined in SPRITE.HPP requires C++ to compile.
#endif

#include <alloc.h>
#include <mem.h>
#include "include\vmode.h"
#include "include\boolean.h"  // Rem this line if your compiler supports 'bool'

class Sprite {
protected:
    int X, Y;
    int lastX, lastY;

    int width, height;

    unsigned int totalNumOfImages;
    unsigned int imageIndex;
    unsigned char far **images;
    bool imagesOwned;

    unsigned char far *background;

    bool visible;
    bool usesTransparentPixels;
    bool initOk;

    // Shared data for all Sprites
    static unsigned int spriteCount;
    static unsigned char far *displayMem;

public:
    // Member functions

    // Constructors and destructor
    Sprite(int w=0, int h=0, unsigned int n=0,
               bool useBG=true, bool ownImages=true);  // Default constructor
    Sprite(const Sprite &s);                           // Copy constructor
    virtual ~Sprite(void);                             // Destructor

    // Operator overloaders
    Sprite &operator=(const Sprite &s);        // Assigment operator

    // General member functions

    // Gets or sets the X and Y coordinates
    int GetX(void) {return X;}
    int GetY(void) {return Y;}
    void SetX(int x) {lastX = X; X = x;}
    void SetY(int y) {lastY = Y; Y = y;}

    // Increments and Decrements the X and Y coordinates by 1
    void IncrX(void) {lastX = X++;}
    void IncrY(void) {lastY = Y++;}
    void DecrX(void) {lastX = X--;}
    void DecrY(void) {lastY = Y--;}

    // Adds an offset to the X and Y coordinates
    void AddX(int xOffset) {lastX = X; X += xOffset;}
    void AddY(int yOffset) {lastY = Y; Y += yOffset;}

    // Updates the image position on screen.
    void Move(int x, int y, unsigned int i);
    void Move(int x, int y)
    {
        Move(x, y, imageIndex);
    }
    void MoveRelative(int xOffset, int yOffset)
    {
        Move(X + xOffset, Y + yOffset, imageIndex);
    }
    void MoveRelative(int xOffset, int yOffset, unsigned int i)
    {
        Move(X + xOffset, Y + yOffset, i);
    }

    // Gets and Sets the dimensions of the images
    int GetWidth(void)  {return width;}
    int GetHeight(void) {return height;}
    bool SetDimensions(int w, int h, bool useBackground);

    // Gets or sets the number of images in use.
    unsigned int GetTotalNumOfImages(void) {return totalNumOfImages;}
    bool SetTotalNumOfImages(unsigned int n);

    // Gets or Sets the image index value.
    unsigned int GetImageIndex(void) {return imageIndex;}
    void SetImageIndex(unsigned int i)
    {
        if (i < totalNumOfImages)
            imageIndex = i;
    }
    void DisplayImage(unsigned int i);

    // Gets or Sets the currently indexed image.
    const unsigned char far *GetIndexedImageAddress(unsigned int i)
    {
        if (i < totalNumOfImages)
            return images[i];
        else
            return NULL;
    }
    const unsigned char far *GetIndexedImageAddress(void)
    {
        return GetIndexedImageAddress(imageIndex);
    }
    void SetIndexedImage(unsigned char far *img, unsigned int i);
    void SetIndexedImage(unsigned char far *img)
    {
        SetIndexedImage(img, imageIndex);
    }

    // Gets the ownership status of the images.
    bool OwnsImages(void) {return imagesOwned;}

    // Gets and Sets the visibility status of the images.
    bool IsVisible(void) {return visible;}
    virtual void Hide(void);
    virtual void Show(void);

    // Gets and Sets the transparency flag status.
    bool GetTransparencyFlag(void) {return usesTransparentPixels;}
    void SetTransparencyFlag(bool t) {usesTransparentPixels = t;}

    // Gets the initialization status.
    bool InitOk(void) {return initOk;}

    static unsigned int GetSpriteCount(void) {return spriteCount;}

    static unsigned char far *GetDisplayAddress(void) {return displayMem;}
    static void SetDisplayAddress(unsigned char far *buffer)
    {
        if (buffer == NULL)
            displayMem = videoMem;
        else
            displayMem = buffer;
    }
};

#endif  /* _SPRITE_HPP */
