/* JOYBIOS.C -- Reads joystick pot values and buttons status.
 *
 * Using BIOS routines for both.
 *
 * by Gary Neal, Jr -- garyneal@geocities.com
 */

#include <dos.h>             /* Needed for int86 functionality */
#include <stdio.h>           /* Definition of NULL */
#include <conio.h>           /* Console specific IO */
#include "include\vmode.h"   /* Our video mode functions */

/* Constants for joystick control */
#define JsA_Xaxis   0x01
#define JsA_Yaxis   0x02
#define JsB_Xaxis   0x04
#define JsB_Yaxis   0x08

#define JsA_Button1 0x10
#define JsA_Button2 0x20
#define JsB_Button1 0x40
#define JsB_Button2 0x80

/* Function prototypes */
int JoystickButtonPressed(int buttonMask);
void GetJoystickPots(int *XA, int *YA, int *XB, int *YB);

/* Main function */
int main(void)
{
    int X, Y, C = 1;      /* Pixel attributes */
    int ReleaseFlag = 1;  /* Color change button released status */

    if (!SetVideoMode(VGA256)) return 1;  /* Exit if no VGA */

    while (kbhit()) getch();              /* Clear the keyboard */

    /* Run until keyboard key pressed or button 2 pressed */
    while (!JoystickButtonPressed(JsA_Button2) && !kbhit()) {

        GetJoystickPots(&X, &Y, NULL, NULL);  /* Return joystick pot value */

        X %= ScreenWidth;                     /* Set coordinates for pixel */
        Y %= ScreenHeight;

        /* Check to see if button 1 was released */
        if (ReleaseFlag) {
            /* If it was released, change color if it is being pressed */
            if (JoystickButtonPressed(JsA_Button1)) {
                C = ++C % ScreenColors;       /* Increment color value */
                ReleaseFlag = 0;              /* Set button as being pressed */
            }
        } else
            /* Set release state according to button status */
            ReleaseFlag = !JoystickButtonPressed(JsA_Button1);

        SetPixel(X, Y, C);                    /* Plot pixel on screen */
    }

    while (kbhit()) getch();                  /* Clear the keyboard */
    SetVideoMode(TEXT_MODE);                  /* Return to text mode */
    return 0;
}

/* JoystickButtonPressed function */
int JoystickButtonPressed(int buttonMask)
{
    union REGS regs;                       /* Define register variable */

    regs.h.ah = 0x84;                      /* Joystick functions */
    regs.x.dx = 0x00;                      /* Get button status */

    int86(0x15, &regs, &regs);             /* Call BIOS to get bits */

    return ((~regs.h.al) & buttonMask);    /* Invert and return status bits */
}

/* GetJoystickPots function */
void GetJoystickPots(int *XA, int *YA, int *XB, int *YB)
{
    union REGS regs;                       /* Define register variable */

    regs.h.ah = 0x84;                      /* Joystick functions */
    regs.x.dx = 0x01;                      /* Get button status */

    int86(0x15, &regs, &regs);             /* Call BIOS to get bits */

    if (XA != NULL) *XA = regs.x.ax;       /* Joystick A, X-axis */
    if (YA != NULL) *YA = regs.x.bx;       /* Joystick A, Y-axis */
    if (XB != NULL) *XB = regs.x.cx;       /* Joystick B, X-axis */
    if (YB != NULL) *YB = regs.x.dx;       /* Joystick B, Y-axis */
}
