/* JOYTEST.C -- Illustrates how joysticks are used in some games.
 * by Gary Neal, Jr. -- garyneal@geocities.com
 */

#include <conio.h>                   /* Console specific IO */
#include "include\vmode.h"           /* Video functions */
#include "include\joystick.h"        /* Our joystick functions */

/* Function prototype */
void CallibrateJoystick(int joystickNum);

/* Global variables */
struct {
    unsigned short Min;              /* Minimum pot value */
    unsigned short Max;              /* Maximum pot value */
} joyPotLimit[4];                   /* Pot value limits */

/* Begin main program */
int main(int argc, char *argv[])
{
    int X = 160, Y = 100, C = 1;     /* Pixel attributes */
    int buttonReleased = 1;          /* Button released flag */

    joystickDirectAccess = 1;        /* Use direct access for testing */
    Joyin(1);                        /* Test for joystick pots */
    if (joyPotVal[JsA_Xaxis] == 0 && joyPotVal[JsA_Yaxis] == 0) {
        cputs("No joystick detected...\r\nAborting...\r\n");
        return 1;                    /* Quit if joystick pots not detected */
    }

    /* Set access according to input parameter */
    if (argc >= 2 && *argv[1] == '1')
        joystickDirectAccess = 1;
    else
        joystickDirectAccess = 0;

    CallibrateJoystick(0);           /* Callibrate Joystick A */

    if (!SetVideoMode(VGA256)) {
        cputs("Could not initialize VGA system...\r\n");
        return 1;
    }

    while (kbhit()) getch();         /* Clear the keyboard */

    /* End program at the stroke of a key */
    while (!kbhit()) {
        Joyin(0);                    /* Get joystick pot values */

        SetPixel(X, Y, 0);           /* Erase pixel from screen */

        /* Get horizontal coordinate and adjust X-coordinate */
        if (joyPotVal[JsA_Xaxis] < joyPotLimit[JsA_Xaxis].Min) {
            if (--X < 0) X = 0;
        } else if (joyPotVal[JsA_Xaxis] > joyPotLimit[JsA_Xaxis].Max) {
            if (++X >= ScreenWidth) X = ScreenWidth - 1;
        }

        /* Get vertical coordinate and adjust Y-coordinate */
        if (joyPotVal[JsA_Yaxis] < joyPotLimit[JsA_Yaxis].Min) {
            if (--Y < 0) Y = 0;
        } else if (joyPotVal[JsA_Yaxis] > joyPotLimit[JsA_Yaxis].Max) {
            if (++Y >= ScreenHeight) Y = ScreenHeight - 1;
        }

        /* Change color if button is pressed */
        if (buttonReleased) {
            if (JoystickButtonPressed(JsAbutton1)) {
                C = ++C % ScreenColors;
                buttonReleased = 0;
            }
        } else
            buttonReleased = !JoystickButtonPressed(JsAbutton1);

        /* Redraw pixel on screen */
        if (C == 0) SetPixel(X, Y, 15);
        SetPixel(X, Y, C);
    }

    /* Clean up and return */
    while (kbhit()) getch();         /* Clear the keyboard */
    SetVideoMode(TEXT_MODE);
    return 0;
}

void CallibrateJoystick(int joystickNum)
{
    int buttonMask, potBase;

    buttonMask = 0x30 << ((joystickNum & 1) << 1);
    potBase    = (joystickNum & 1) << 1;

    cprintf("Callibrating Joystick %c", ((joystickNum & 1) ? 'B' : 'A'));
    while (kbhit()) getch();         /* Clear the keyboard */
    cputs("\r\nMove joystick to far upper-left and press a button.");
    while (!JoystickButtonPressed(buttonMask) && !kbhit()) {
        Joyin(0);                    /* Get joystick pots */
        joyPotLimit[potBase].Min     = joyPotVal[potBase]     + 20;
        joyPotLimit[potBase + 1].Min = joyPotVal[potBase + 1] + 20;
    }
    while (JoystickButtonPressed(buttonMask) && !kbhit());
    while (kbhit()) getch();         /* Clear the keyboard */
    cputs(" [OK]\r\nMove joystick far lower-right and press a button.");
    while (!JoystickButtonPressed(buttonMask) && !kbhit()) {
        Joyin(0);                    /* Get joystick pots */
        joyPotLimit[potBase].Max     = joyPotVal[potBase]     - 20;
        joyPotLimit[potBase + 1].Max = joyPotVal[potBase + 1] - 20;
    }
    while (JoystickButtonPressed(buttonMask) && !kbhit());
    while (kbhit()) getch();         /* Clear the keyboard */
    cputs(" [OK]");
}
