/******************************************************************************
 *                            STANDARD SPRITE CLASS                           *
 *                              by Gary Neal, Jr.                             *
 *               garyneal_71@yahoo.com       garyneal@geocities.com           *
 ******************************************************************************/

#include "include\csprite.hpp"

/******************************************************************************/

ClipSprite::ClipSprite(int w, int h, unsigned int n, bool useBG,
                       bool ownImages) : Sprite(w, h, n, useBG, ownImages)
{
    minX = minY = 0;
    maxX = ScreenWidth  - 1;
    maxY = ScreenHeight - 1;

    startXoff = startYoff = lenXcopy = lenYcopy = 0;
}

/******************************************************************************/

ClipSprite::ClipSprite(const ClipSprite &s) : Sprite(s)
{
    minX = s.minX;
    minY = s.minY;
    maxX = s.maxX;
    maxY = s.maxY;

    startXoff = s.startXoff;
    startYoff = s.startYoff;
    lenXcopy  = s.lenXcopy;
    lenYcopy  = s.lenYcopy;
}

/******************************************************************************/

ClipSprite &ClipSprite::operator=(const ClipSprite &s)
{
    ClipSprite::~ClipSprite();           // Destroy this sprite.
    ClipSprite::ClipSprite(s);           // Initialize to passed sprite.
    return (*this);                      // Return reference.
}

/******************************************************************************/

void ClipSprite::Hide(void)
{
    int i;
    unsigned char far *src;
    unsigned char far *dest;

    if (!visible) return;

    if (background != NULL && lenXcopy && lenYcopy) {
        src  = &background[startYoff * width + startXoff];
        dest = &displayMem[((Y+startYoff)<<8)+((Y+startYoff)<<6)+X+startXoff];

        for (i = 0; i < lenYcopy; i++) {
            _fmemcpy(dest, src, lenXcopy);
            src  += width;
            dest += ScreenWidth;
        }
    }

    visible = false;
}

/******************************************************************************/

void ClipSprite::Show(void)
{
    int i, t;
    unsigned char far *src;
    unsigned char far *dest;

    if (visible) return;

    visible = true;

    if (images == NULL || images[imageIndex] == NULL) return;

    if (X < minX)
        startXoff = minX - X;
    else
        startXoff = 0;

    if (startXoff < width) {
        if (X > 0 && (short)(X + width - 1) < 0) {
            lenXcopy = 0;
            return;
        } else if (X + width - 1 > maxX) {
            lenXcopy = maxX - (X + startXoff) + 1;
        } else {
            lenXcopy = width - startXoff;
        }
        if (lenXcopy < 0) {
            lenXcopy = 0;
            return;
        }
    } else {
        lenXcopy = 0;
        return;
    }

    if (Y < minY)
        startYoff = minY - Y;
    else
        startYoff = 0;

    if (startYoff < height) {
        if (Y > 0 && (short)(Y + height - 1) < 0) {
            lenYcopy = 0;
            return;
        } else if (Y + height - 1 > maxY) {
            lenYcopy = maxY - (Y + startYoff) + 1;
        } else {
            lenYcopy = height - startYoff;
        }
        if (lenYcopy < 0) {
            lenYcopy = 0;
            return;
        }
    } else {
        lenYcopy = 0;
        return;
    }

    if (background != NULL) {
        src  = &displayMem[((Y+startYoff)<<8)+((Y+startYoff)<<6)+X+startXoff];
        dest = &background[startYoff * width + startYoff];

        for (i = 0; i < lenYcopy; i++) {
            _fmemcpy(dest, src, lenXcopy);
            src  += ScreenWidth;
            dest += width;
        }
    }

    src  = &images[imageIndex][startYoff * width + startXoff];
    dest = &displayMem[((Y+startYoff)<<8)+((Y+startYoff)<<6)+X+startXoff];

    if (usesTransparentPixels) {
        for (i = 0; i < lenYcopy; i++) {
            for (t = 0; t < lenXcopy; t++)
                if (src[t]) dest[t] = src[t];
            src  += width;
            dest += ScreenWidth;
        }
    } else {
        for (i = 0; i < lenYcopy; i++) {
            _fmemcpy(dest, src, lenXcopy);
            src  += width;
            dest += ScreenWidth;
        }
    }
}

/******************************************************************************/

void ClipSprite::SetDisplayArea(int min_x, int min_y, int max_x, int max_y)
{
    if (min_x < 0)
        min_x = 0;
    if (min_y < 0)
        min_y = 0;

    if (max_x <= 0 || max_x >= ScreenWidth)
        max_x = ScreenWidth - 1;
    if (max_y <= 0 || max_x >= ScreenHeight)
        max_y = ScreenHeight - 1;

    minX = min_x;
    minY = min_y;
    maxX = max_x;
    maxY = max_y;
}

/******************************************************************************/

