/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * Library source file - VMODE.C                                             *
 * by Gary Neal, Jr. -- garyneal@geocities.com                               *
 *                                                                           *
 * Allows us to set the video mode between modes 13h and the text mode.      *
 * Provides information about mode 13h (height, width, and no of colors).    *
 * Allows us to set pixels and blank the screen.                             *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "include\vmode.h"  /* Header file for this library */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * SetVideoMode  ----  Sets video card to any of the standard video modes.   *
 *                                                                           *
 * This function changes the display using any of the standard video modes   *
 * supported by the BIOS.                                                    *
 *                                                                           *
 * This function has a return value of false (0) if the video mode failed to *
 * change, true (1) otherwise.  Return value can be ignored for compatibility*
 * with older code written with these library files.                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int SetVideoMode(int mode)
{
    union REGS regs;                  /* Define register variable */

    regs.x.ax = mode;                 /* Set video mode */
    regs.h.ah = 0;                    /* Zero Most Significant Byte */
    int86(0x10, &regs, &regs);        /* Call interrupt to change the mode */
    return (mode == GetVideoMode());  /* Return change condition */
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * GetVideoMode  ----  Gets the current video mode from the BIOS.            *
 *                                                                           *
 * This function returns the display mode in use by the BIOS.                *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int GetVideoMode(void)
{
    union REGS regs;              /* Define register variable */

    regs.x.ax = 0xF00;            /* Get current video mode */
    int86(0x10, &regs, &regs);    /* Call interrupt to get the mode */

    regs.h.ah = 0;
    return (regs.x.ax);           /* Return Current Video Mode */
}
