/* Library source file - VMODE.C
 * by Gary Neal, Jr. -- garyneal@geocities.com
 *
 * Allows us to set the video mode between modes 13h and the text mode.
 * Provides information about mode 13h (height, width, and no of colors).
 * Allows us to set pixels and blank the screen.
 *
 * This is the alternate source file to the source file VMODE.C
 * you may use this in place of VMODE.C by renaming this file to VMODE.C
 * and it's corresponding header, VMODEALT.H, to VMODE.H.
 *
 * To use it in the library, you must compile this to OBJ and replace the
 * VMODE.OBJ file in the GAMELIB.LIB file in the LIB directory.
 *
 * For details, go to http://www.geocities.com/SiliconValley/Park/7113/
 */

#include "include\vmode.h"  /* Header file for this library */

/* Sets a global pointer to video memory */
unsigned char far *videoMem = (unsigned char far *)0xA0000000L;

/* Sets or changes the video display mode */
int SetVideoMode(int mode)
{
    union REGS regs;    /* Define register variable */

    regs.x.ax = mode;   /* Set video mode */

    /* Call interrupt to complete the task */
    int86(0x10, &regs, &regs);

    return (mode == GetVideoMode());  /* Return change condition */
}

/* Gets the currently displayed video mode */
int GetVideoMode(void)
{
    union REGS regs;              /* Define register variable */

    regs.x.ax = 0xF00;            /* Get current video mode */
    int86(0x10, &regs, &regs);    /* Call interrupt to get the mode */

    return (regs.h.al);           /* Return Current Video Mode */
}

/* Sets a single pixel in Mode 13H */
void SetPixel(int X, int Y, unsigned char C)
{
    videoMem[((Y << 8) + (Y << 6)) + X] = C;
}

/* Sets all pixels in Mode 13H */
void FillScreen(unsigned char C)
{
    _fmemset(videoMem, C, 64000);
}
